#include <avr/io.h>

void UARTInit( unsigned int ubrr)
{
    /* Baud rate settings */
    UBRRH = (unsigned char)(ubrr>>8);
    UBRRL = (unsigned char)ubrr;
    /* Enabling transmitter and receiver */
    UCSRB = (1<<RXEN)|(1<<TXEN);
    /* Frame format settings: 8data, 2stop bit */
    UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
}

void UARTTransmit( unsigned char data )
{
    /* We wait until the buffer of the transmitter gets empty. */
    while ( !( UCSRA & (1<<UDRE)) )
	{
		/* Empty loop. We do nothing else, just wait. */
	}
    /* We put the data into the buffer.
	 * The sending is solved by the microcontroller's HW. */
    UDR = data;
}

void UARTTransmitString(char string[])
{
    unsigned char iterator;
    for(iterator = 0; string[iterator] != '\0'; iterator++)
    {
        UARTTransmit(string[iterator]);
    }
}

int main(void)
{
    char helloWorld[] = "Hello World!";
	
    /* Enabling PORTA pull-up resistors on the inputs
     * PA0 output should have logic 1 level (5V), on this pin is the status LED
	 * The other pins should be configured as inputs */
    PORTA = 0xFF;
    DDRA = 0x01;

    /* Enabling PORTB pull-up resistors, and all pins should be configured as inputs */
    PORTB = 0xFF;
    DDRB = 0x00;

     /* Enabling PORTC pull-up resistors, and all pins should be configured as inputs */
    PORTC = 0xFF;
    DDRC = 0x00;

    /* Enabling PORTD pull-up resistors, and all pins should be configured as inputs */
    PORTD = 0xFF;
    DDRD = 0x00;
    
    UARTInit(103);
    
    UARTTransmitString(helloWorld);
        
    while(1)
    {
		/* Infinite loop */
    }
	
	/* We never get here, but syntactically it fits here */
	return 0;
}

